#include <algorithm>

void nn_H4b(int ntrain=220)
{
    if (!gROOT->GetClass("TMultiLayerPerceptron")) {
        gSystem->Load("libMLP");
    }

    TFile *f = new TFile("nn.root");
    TTree *snb = (TTree*) gDirectory->Get("NN");
//  # of input node = 7 (# of variables)
//  # of first hidden layer = 13
//  # of second hidden layer = 4
//  # of output layer = 1 (variable "type")
//    TMultiLayerPerceptron *mlp = new TMultiLayerPerceptron("dphi12,dphi13,dphi23,ht,m12,m23,pt_b1b2:13:4:@type","weight",snb,"Entry$%2","Entry$/2");
    TMultiLayerPerceptron *mlp = new TMultiLayerPerceptron("dphi12,dphi13,dphi23,ht,m12,m23,pt_b1b2:13:4:type","weight",snb);
// Alternatively, dr12,dr13,dr23,ht_3b can be used.
    mlp->SetLearningMethod(TMultiLayerPerceptron::kBFGS);
    mlp->Train(ntrain,"text,graph,update=10");

    TMLPAnalyzer ana(mlp);
    // Initialisation
    ana.GatherInformations();
    // output to the console
    ana.CheckNetwork();

    TCanvas* c1 = new TCanvas("c1","Network Analysis",1600,1600);
    // shows how each variable influences the network
    ana.DrawDInputs();
    c1->Print("nn_analysis.eps");

    TCanvas* c2 = new TCanvas("c2","Network Structure",2500,1600);
    // shows the network structure
    mlp->Draw();
    c2->Print("nn_structure.eps");

    TCanvas* c3 = new TCanvas("c3","Sig/Bkgd Separation 1",1600,1600);
    // draws the resulting network
    ana.DrawNetwork(0,"type==1","type==0");
    c3->Print("nn_snb1.eps");

    TCanvas* c4 = new TCanvas("c4","Sig/Bkgd Separation 2",1600,1600);
    // Use the NN to plot the results for each sample
    // This will give approx. the same result as DrawNetwork.
    // All entries are used, while DrawNetwork focuses on 
    // the test sample. Also the xaxis range is manually set.
    TH1F *bg = new TH1F("bg", "NN output", 50, -.5, 1.5);
    TH1F *sig = new TH1F("sig", "NN output", 50, -.5, 1.5);
    Double_t params[7];
    int NN_type;
    snb->SetBranchAddress("NN",&NN_type);
    for ( int i=0; i<snb->GetEntries(); i++) {
        snb->GetEntry(i);
        params[0] = dphi12;
        params[1] = dphi13;
        params[2] = dphi23;
        params[3] = ht;
        params[4] = m12;
        params[5] = m23;
        params[6] = pt_b1b2;
        if ( NN_type==0 )
            bg->Fill(mlp->Evaluate(0, params));
        else if ( NN_type==1 )
            sig->Fill(mlp->Evaluate(0,params));
    }
    bg->SetLineColor(kBlue);
    bg->SetFillStyle(3008);   bg->SetFillColor(kBlue);
    sig->SetLineColor(kRed);
    sig->SetFillStyle(3003); sig->SetFillColor(kRed);
    bg->SetStats(0);
    sig->SetStats(0);
    bg->Scale(1./bg->GetEntries());
    sig->Scale(1./sig->GetEntries());
    bg->Draw();
    sig->Draw("same");
    TLegend *legend = new TLegend(.75, .80, .95, .95);
    legend->AddEntry(bg, "Background (Fakes)");
    legend->AddEntry(sig, "Signal (Higgs)");
    legend->Draw();
    c4->Print("nn_snb2.eps");
}
